let LOCALE_DIR = "/opt/jhbuild/share/locale";
let STYLE_DIR = "/opt/jhbuild/share/gnome-documents/style/";
let ICONS_DIR = "/opt/jhbuild/share/gnome-documents/icons/";
let RESOURCE_DIR = "/opt/jhbuild/share/gnome-documents";
